----------------------------------------------
--		Serenity Now						--
--	By Shampow(Calahil) on Archimonde-US	--
----------------------------------------------
-- r78
-- 1  


local addonName, SNEngine = ...
local SN = LibStub("AceAddon-3.0"):NewAddon(addonName, "AceEvent-3.0", "AceTimer-3.0",  "AceConsole-3.0")
local AceConfig = LibStub("AceConfig-3.0")
local AceConfigDialog = LibStub("AceConfigDialog-3.0")
local AceDB = LibStub("AceDB-3.0")
local AceDBOptions = LibStub("AceDBOptions-3.0")

local L = LibStub("AceLocale-3.0"):GetLocale("SerenityNow")

local timers = LibStub("LibFancyBar-1.0")
local SNFrames = {}
local LSM = LibStub("LibSharedMedia-3.0")
local ISPCT = LibStub("LibGroupInSpecT-1.0")

local collectgarbage = collectgarbage
local InterfaceOptionsFrame_OpenToCategory = InterfaceOptionsFrame_OpenToCategory
local print = print
local format = format

SN.dispel = LibStub("LibDispellable-1.0")
SN.ver = GetAddOnMetadata("SerenityNow", "Version")
SN.addonName = addonName

SN.playerClass = select(2, UnitClass("player"))
SN.playerName = UnitName("player")
SN.RaidMemberData = {}
-- How often the OnUpdate code will run (in seconds)
--SN_UpdateInterval = 1.0 
SN.UpdateIntervalDispel = 1.0

local SNmedia = {
	["media"] = {
		["normTex"] = LSM:Fetch("statusbar","SNNorm"),
		["minTex"] = LSM:Fetch("statusbar", "SNMin"),
		["glossTex"] = LSM:Fetch("statusbar", "SNGloss"),
		["blankTex"] = LSM:Fetch("background","SNBlank"),
		["draghandle"] = LSM:Fetch("border", "SNDrag"),
		["borderTex"] = LSM:Fetch("border", "SNBorder"),
		["font"] = LSM:Fetch("font","SNFont")
	},
	["power"] = {
		{ 0.31, 0.45, 0.63 }, --Mana
		{ 0.78, 0.25, 0.25 }, --Rage
		{ 0.71, 0.43, 0.27 }, --Focus
		{ 0.65, 0.63, 0.35 }, --Energy
		{ 0, 0.82, 1 }, --Runic Power
	},
	["classResources"] = {
		["DEATHKNIGHT"] = {
			{ 1, 0, 0 },
			{ 0, .5, 0 },
			{ 0, 1, 1 },
			{ .9, 1, 1 },				
		},
		["DRUID"] = {
			{ .30, .52, .90 },
			{ .80, .82, .60 },
		},
		["MAGE"] = { 0, .61, 1 },
		["MONK"] = {
			{ .57, .63, .35 },
			{ .47, .63, .35 },
			{ .37, .63, .35 },
			{ .27, .63, .33 },
			{ .17, .63, .33 },			
		},
		["PALADIN"] = { 1, 1, 0 },
		["PRIEST"] = { 1, 1, 1 },
		["ROGUE"] = {
			{ .69, .31, .31 },
			{ .69, .31, .31 },
			{ .65, .63, .35 },
			{ .65, .63, .35 },
			{ .33, .59, .33 },
		},
		["WARLOCK"] = {
			{ .58, .50, .78 },
			{ .58, .50, .78 },
			{ .90, .37, .37 },			
		},
	},
}

--Serenity Now's Engine Variables
SNEngine[1] = SN			--Addon
SNEngine[2] = L				--Locale
SNEngine[3] = SNFrames		--Frame Table
SNEngine[4] = timers		--LibFancyBars
SNEngine[5] = SNmedia		--Shared Media

_G[addonName] = SNEngine


function SN:OnInitialize()
	--Setup the clc 
	SN:RegisterChatCommand("sn", "Cmds")
	--Lets make the welcome message more visible
	SN:ScheduleTimer("Ver", 10)
	SN:Initialize()
	
	SN.data.profile.frames.lock = true
	
	--collect garbage from startup
	collectgarbage("collect")

	ISPCT.RegisterCallback(SN, "GroupInSpecT_Update", "RosterUpdate")
	ISPCT.RegisterCallback(SN, "GroupInSpecT_Remove", "RosterRemove")
end

function SN:OnEnable()
	for name, module in SN:IterateModules() do
		if SN.data.profile[name].enable then
			SN:EnableModule(name)
		else
			SN:DisableModule(name)
		end
	end
end

function SN:Ver()
	print(format(L["Welcome to |cff0070deSerenity Now|r "]..SN.ver))
end

function SN:Cmds(input)
	local v1, v2 = input:match("^(%S*)%s*(.-)$")
	v1 = v1:lower()

	if (v1 == "") then	
		print(L["|cff0070deSerenity Now|r "]..SN.ver)
		SN:Print(L["/sn |cff0070deconfig|r - open the config frame."])
		SN:Print(L["/sn |cff0070delock|r - unlock/lock the frames."])
		SN:Print(L["/sn |cff0070dereset|r - reset the frames."])

	elseif (v1 == L["options"]) or (v1 == L["config"]) then
		
		if InCombatLockdown() then 
			SN:Print(L["Can not do that while in combat"])
		else
			--InterfaceOptionsFrame_OpenToCategory("Serenity Now")
			AceConfigDialog:Open("SN")
		end
	elseif (v1 == L["lock"]) or (v1 == L["unlock"]) then
		if SN.data.profile.frames.lock then
			if InCombatLockdown() then 
				SN:Print(L["Can not do that while in combat"])
			else
				SN:MoveFrames(false)
				SN.data.profile.frames.lock = false
				SN:Print(L["Frames Unlocked"])
			end
		elseif not SN.data.profile.frames.lock then
			if InCombatLockdown() then 
				SN:Print(L["Can not do that while in combat"])
			else
				SN:MoveFrames(true)
				SN.data.profile.frames.lock = true
				SN:Print(L["Frames Locked"])
			end
		end
	elseif (v1 == L["reset"]) or (v1 == L["defaults"]) then
		if InCombatLockdown() then 
			SN:Print(L["Can not do that while in combat"])
		else
			SN:ResetAnchors()
		end
	else
		SN:Print(L["Command Unknown."])
	end
end

function SN:Debug(...)
	if SN.data.profile.debug.enable then
		SN:Print(...)
	end
end