﻿-----------------------------------------------------------------------------------------
--Debuffs Group Frame--------------------------------------------------------------------
-----------------------------------------------------------------------------------------
local SN, L, F, T, M, _ = unpack(select(2,...))
local LSM = LibStub("LibSharedMedia-3.0")
local MODNAME = "debuffs"
local DB = SN:NewModule(MODNAME, "AceEvent-3.0")

local UnitDebuff = UnitDebuff
local UnitGUID = UnitGUID
local UnitName = UnitName
local UnitIsEnemy = UnitIsEnemy
local GetTime = GetTime
local IsAddOnLoaded = IsAddOnLoaded
local pairs = pairs
local unpack = unpack
local bit_band = bit.band
local COMBATLOG_OBJECT_REACTION_FRIENDLY = COMBATLOG_OBJECT_REACTION_FRIENDLY
local COMBATLOG_OBJECT_REACTION_NEUTRAL = COMBATLOG_OBJECT_REACTION_NEUTRAL

local debuffs

local function getOptions()
	debuffs = {
		type = "group",
		name = L["Debuffs"],
		args = {
			enable = {
				order = 0,
				type = "toggle",
				name = L["Enable Debuff Bars"],
				get = function(info)
					return SN.data.profile.debuffs[info[#info]]
				end,
				set = function(info, value)
					SN.data.profile.debuffs[info[#info]] = value

					if value then
						SN:EnableModule(MODNAME)
					else
						SN:DisableModule(MODNAME)
					end
				end,
			},
			visible = {
				order = 1,
				type = "range",
				name = L["Number of Debuff Bars Visible"],
				hidden = function() return not SN.data.profile.cooldown.enable end,
				min = 1, max = 8, step = 1,
				get = function(info)
					return SN.data.profile.debuffs[info[#info]] 
				end,
				set = function(info, value)
					SN.data.profile.debuffs[info[#info]] = value
					F.debuffs:SetVisibleBars(value)
				end,
			},
			h7 = {
				order = 2,
				type = "header",
				name = "Dimensions",
				hidden = function() return not SN.data.profile.debuffs.enable end,
			},
			width = {
				order = 4,
				type = "range",
				name = L["Width of Debuff Bars"],
				hidden = function() return not SN.data.profile.debuffs.enable end,
				min = 50, max = 500, step = 1,
				get = function(info)
					return SN.data.profile.debuffs[info[#info]] 
				end,
				set = function(info, value)
					SN.data.profile.debuffs[info[#info]] = value
					F.debuffs:SetWidth(value)
				end,
			},
			height = {
				order = 5,
				type = "range",
				name = L["Height of Debuff Bars"],
				hidden = function() return not SN.data.profile.debuffs.enable end,
				min = 12, max = 40, step = 1,
				get = function(info)
					return SN.data.profile.debuffs[info[#info]] 
				end,
				set = function(info, value)
					SN.data.profile.debuffs[info[#info]] = value
					if F.debuffs.bars ~= nil then
						local bars = F.debuffs.bars
						for i in pairs(bars) do
							i:SetHeight(value)
						end
					end
				end,
			},
			yoffset = {
				order = 6,
				type = "range",
				name = L["Y Offset of Debuff Bars"],
				hidden = function() return not SN.data.profile.cooldown.enable end,
				order = 7,
				min = 0, max = 6, step = 1,
				get = function(info)
					return SN.data.profile.cooldown[info[#info]]
				end,
				set = function(info, value)
					SN.data.profile.debuffs[info[#info]] = value
					F.debuffs:SetSpacing(SN.data.profile.debuffs.xoffset, value)
				end,
			},
			h8 = {
				order = 8,
				type = "header",
				name = "Font",
				hidden = function() return not SN.data.profile.debuffs.enable end,
			},
			font = {
				order = 9,
				type = "select",
				dialogControl = 'LSM30_Font',
				name = L["Font"],
				hidden = function() return not SN.data.profile.debuffs.enable end,
				values = AceGUIWidgetLSMlists.font,
				get = function(info) return SN.data.profile.debuffs[info[#info]] end,--selectedClass end,
				set = function(info, value)
					SN.data.profile.debuffs[info[#info]] = value
					if F.debuffs.bars ~= nil then
						local bars = F.debuffs.bars
						for i in pairs(bars) do
							i:SetLabelFont(LSM:Fetch("font", value), SN.data.profile.debuffs.fontsize)
							i:SetDurationFont(LSM:Fetch("font", value), SN.data.profile.debuffs.fontsize)
						end
					end
				end,
			},
			fontsize = {
				order = 10,
				type = "range",
				name = L["Font Size of Debuff Bars"],
				hidden = function() return not SN.data.profile.debuffs.enable end,
				min = 10, max = 24, step = 1,
				get = function(info)
					return SN.data.profile.debuffs[info[#info]] 
				end,
				set = function(info, value)
					SN.data.profile.debuffs[info[#info]] = value
					if F.debuffs.bars ~= nil then
						local bars = F.debuffs.bars
						for i in pairs(bars) do
							i:SetLabelFont(LSM:Fetch("font", SN.data.profile.debuffs.font), value)
							i:SetDurationFont(LSM:Fetch("font", SN.data.profile.debuffs.font), value)
						end
					end
				end,
			},
			h9 = {
				order = 11,
				type = "header",
				name = L["Texture/Misc"],
				hidden = function() return not SN.data.profile.debuffs.enable end,
			},
			texture = {
				order = 12,
				type = "select",
				dialogControl = 'LSM30_Statusbar',
				name = L["Texture"],
				hidden = function() return not SN.data.profile.debuffs.enable end,
				values = AceGUIWidgetLSMlists.statusbar,
				get = function(info) return SN.data.profile.debuffs[info[#info]] end,
				set = function(info, value)
					SN.data.profile.debuffs[info[#info]] = value
					if F.debuffs.bars ~= nil then
						local bars = F.debuffs.bars
						for i in pairs(bars) do
							i:SetTexture(LSM:Fetch("statusbar", value))
						end
					end
				end,
			},
			grow = {
				order = 13,
				type = "select",
				name = L["Grow Direction"],
				hidden = function() return not SN.data.profile.debuffs.enable end,
				values = {
					["topleft"] = "Up",
					["bottomleft"] = "Down",
				},
				get = function(info)
					return SN.data.profile.debuffs[info[#info]] 
				end,
				set = function(info, value)
					SN.data.profile.debuffs[info[#info]] = value
					F.debuffs:SetStartPoint(value)
				end,
			},
			iconmode = {
				order = 14,
				type = "toggle",
				name = L["Turn on Icon Mode"],
				hidden = function() return not SN.data.profile.debuffs.enable end,
				get = function(info)
					return SN.data.profile.debuffs[info[#info]]
				end,
				set = function(info, value)
					SN.data.profile.debuffs[info[#info]] = value
					if value then
						F.debuffs:SetIconMode(32 * SN.data.profile.debuffs.iconscale, "BOTTOM", 0, 0)
						F.debuffs:SetSpacing(SN.data.profile.debuffs.fontsize / 2, SN.data.profile.debuffs.fontsize + 3)
					else
						F.debuffs:SetIconMode(false)
						F.debuffs:SetSpacing(0, 3)
						F.debuffs:SetWidth(SN.data.profile.debuffs.width)
						if F.debuffs.bars ~= nil then
							local bars = F.debuffs.bars
							for i in pairs(bars) do
								i:SetHeight(SN.data.profile.debuffs.height)
							end
						end
					end
				end,
			},
			iconscale = {
				order = 15,
				type = "range",
				name = L["Icon Scale"],
				min = 0.1, max = 2.0, step = 0.1,
				hidden = function() return not SN.data.profile.debuffs.iconmode or not SN.data.profile.debuffs.enable end,
				get = function(info)
					return SN.data.profile.debuffs[info[#info]] 
				end,
				set = function(info, value)
					SN.data.profile.debuffs[info[#info]] = value
					if SN.data.profile.debuffs.iconmode then
						F.debuffs:SetIconMode(32 * SN.data.profile.debuffs.iconscale, "BOTTOM", 0, 0)
						F.debuffs:SetSpacing(SN.data.profile.debuffs.fontsize / 3, SN.data.profile.debuffs.fontsize + 3)
					end
				end,
			},
		},
	}
	return debuffs
end

function DB:OnInitialize()
	SN:RegisterModuleOptions(MODNAME, getOptions, L["Debuffs"])
	
	F.debuffs = T:NewGroup(SN.data.profile.debuffs.width, "Debuff Frame", debuffs, M.media.draghandle)
	F.debuffs:SetSorting("SN:arg5")
	F.debuffs:SetSpacing(0, SN.data.profile.debuffs.yoffset)
	F.debuffs:SetStartPoint(SN.data.profile.debuffs.grow)
	F.debuffs:SetPoint(unpack(SN.data.profile.frames.debuffs))
	F.debuffs:SetVisibleBars(SN.data.profile.debuffs.visible)

	F.debuffs.name = "Debuffs"
	F.debuffs.db = "debuffs"

	if SN.data.profile.debuffs.iconmode then
		F.debuffs:SetIconMode(32 * SN.data.profile.debuffs.iconscale, "BOTTOM", 0, 0)
		F.debuffs:SetSpacing(SN.data.profile.debuffs.fontsize / 2, SN.data.profile.debuffs.fontsize + 3)
	end
	F.debuffs:SetScript("OnEvent", function(self, event, ...)
		DB:DebuffEvents(self, event, ...)
	end)
end

function DB:OnEnable()
	F.debuffs:Show()		
	--Register our events for the frame
	F.debuffs:RegisterUnitEvent("UNIT_AURA", "target")
	F.debuffs:RegisterEvent("PLAYER_TARGET_CHANGED")
end

function DB:OnDisable()
	F.debuffs:UnregisterAllEvents()
	F.debuffs:Hide()
end

function DB:Lock()
	F.debuffs:Lock(true)
	SN.data.profile.frames.debuffs = { SN:GetPoint(F.debuffs) }
end

function DB:Unlock()
	F.debuffs:Lock(false)
end

function DB:Reset()
	local static = SN.DB.profile.frames
	local temp = SN.data.profile.frames
	F.debuffs:ClearAllPoints()

	F.debuffs:SetPoint(unpack(static.debuffs))
	wipe(temp.debuffs)
end

function DB:ApplySettings()
	F.debuffs:ClearAllPoints()
	SN:EditBars(F.debuffs.db)
end

function DB:DebuffEvents(self, event, ...)

	local bars = self.bars
	if bars then
		local i
		for i in pairs(bars) do
			if i:GetData("SN:arg3") ~= UnitGUID("target") or UnitIsDead("target") then
				i:Stop()
			end
		end
	end
	local i = 1
	while true do
		local name, _, _, _, _, dur, expirationTime, sourceName, _, _, spellId, canApplyAura = UnitDebuff("target", i)
		--Is there a bar for this debuff already?
		if not name then break end
		
		if sourceName == "player" then
			SN:Debug(name.." "..tostring(dur).." "..tostring(expirationTime))
			if not tContains(SN.crowdcontrol, spellId) then
				local dbf = self:Find("SN:arg5", name)
				if not dbf then
					if dur ~= 0 and expirationTime ~= 0 then
						SN:CreateBar(self, spellId, (expirationTime - GetTime()), SN.playerName, UnitGUID("target"), name)
					end
				else
					dbf:SetDuration((expirationTime - GetTime()), dur)
					dbf:Start()
				end
			end
		end
		i = i + 1
	end
end